import { BleScanMode } from "./common";
import { filter, firstValueFrom, map, throwError, timeout, } from "rxjs";
export class BleScanProvider {
    constructor(adapter) {
        this.adapter = adapter;
    }
    isReady() {
        return this.adapter.isReady();
    }
    prepare() {
        return this.adapter.prepare();
    }
    acquireBleScan(options = {}) {
        const safeOptions = fixAcquireOptions(options);
        validateAllUuids(safeOptions.iBeaconUuids);
        return firstValueFrom(this.adapter
            .streamBleScanResults(safeOptions.scanTime, safeOptions.scanMode)
            .pipe(map((result) => filterByUuid(result, safeOptions.iBeaconUuids)), filter((result) => !!result), safeOptions.scanTime > 0
            ? timeout({
                first: safeOptions.scanTime + 100,
                with: () => throwError(() => new Error("BLE scan timed-out")),
            })
            : undefined));
    }
    bleScanStream(options = {}) {
        const safeOptions = fixStreamOptions(options);
        validateAllUuids(safeOptions.iBeaconUuids);
        return this.adapter
            .streamBleScanResults(safeOptions.reportInterval, safeOptions.scanMode)
            .pipe(map((result) => filterByUuid(result, safeOptions.iBeaconUuids)), filter((result) => !!result));
    }
}
function fixAcquireOptions(options) {
    const copy = fixScanningOptions(options);
    if (copy.scanTime === undefined || copy.scanTime < 0) {
        copy.scanTime = 0;
    }
    return copy;
}
function fixStreamOptions(options) {
    const copy = fixScanningOptions(options);
    if (copy.reportInterval === undefined || copy.reportInterval < 0) {
        copy.reportInterval = 0;
    }
    return copy;
}
function fixScanningOptions(options) {
    const copy = Object.assign({}, options);
    if (copy.scanMode === undefined) {
        copy.scanMode = BleScanMode.BALANCED;
    }
    if (copy.iBeaconUuids === undefined) {
        copy.iBeaconUuids = [];
    }
    copy.iBeaconUuids = copy.iBeaconUuids.map((uuid) => uuid.toLowerCase());
    return copy;
}
function filterByUuid(results, uuids) {
    if (uuids.length === 0)
        return results;
    const filteredSeen = results.seen.filter((info) => info.iBeacon && uuids.includes(info.iBeacon.uuid.toLowerCase()));
    if (filteredSeen.length === 0)
        return null;
    return Object.assign(Object.assign({}, results), { seen: filteredSeen });
}
function validateAllUuids(uuids) {
    for (const uuid of uuids) {
        validateUuid(uuid);
    }
}
function validateUuid(uuid) {
    if (!/^[a-f0-9]{8}-[a-f0-9]{4}-[1-9][a-f0-9]{3}-[89aAbB][a-f0-9]{3}-[a-f0-9]{12}$/.test(uuid)) {
        throw new Error(`UUID ${uuid} is not valid!`);
    }
}
//# sourceMappingURL=provider.js.map