import { Observable } from "rxjs";
const MIN_SCAN_INTERVAL = 5000;
export class WifiScanProvider {
    constructor(adapter) {
        this.adapter = adapter;
    }
    isReady() {
        return this.adapter.isReady();
    }
    prepare() {
        return this.adapter.prepare();
    }
    async acquireWifiFingerprint(ensureIsNew = true) {
        return this.adapter.acquireWifiFingerprint(ensureIsNew);
    }
    wifiFingerprintStream(options = {}) {
        const safeOptions = fixStreamOptions(options);
        if (safeOptions.ensureAlwaysNew) {
            return this.wifiFingerprintStreamWithoutDuplicates(safeOptions.grouping, safeOptions.continueOnFailure);
        }
        else {
            return this.wifiFingerprintStreamWithDuplicates(safeOptions.interval, safeOptions.continueOnFailure);
        }
    }
    wifiFingerprintStreamWithoutDuplicates(grouping, continueOnFailure) {
        return new Observable((subscriber) => {
            let unsubscribed = false;
            const performScans = async () => {
                try {
                    for (let i = 0; i < grouping; i++) {
                        const result = await this.adapter.acquireWifiFingerprint(true);
                        if (unsubscribed)
                            break;
                        subscriber.next(result);
                        if (i === grouping - 1)
                            break;
                        await new Promise((resolve) => setTimeout(resolve, MIN_SCAN_INTERVAL));
                    }
                }
                catch (err) {
                    if (continueOnFailure) {
                        console.error("WifiScanProvider ERROR:", err);
                    }
                    else {
                        subscriber.error(err);
                    }
                }
            };
            performScans();
            const intervalId = setInterval(() => {
                performScans();
            }, getIntervalForGrouping(grouping));
            return () => {
                unsubscribed = true;
                clearInterval(intervalId);
            };
        });
    }
    wifiFingerprintStreamWithDuplicates(interval, continueOnFailure) {
        return new Observable((subscriber) => {
            let unsubscribed = false;
            const performScan = async () => {
                try {
                    const result = await this.adapter.acquireWifiFingerprint(false);
                    if (unsubscribed)
                        return;
                    subscriber.next(result);
                }
                catch (err) {
                    if (continueOnFailure) {
                        console.error("WifiScanProvider ERROR:", err);
                    }
                    else {
                        subscriber.error(err);
                    }
                }
            };
            performScan();
            const intervalId = setInterval(() => {
                performScan();
            }, interval);
            return () => {
                unsubscribed = true;
                clearInterval(intervalId);
            };
        });
    }
}
function fixStreamOptions(options) {
    const copy = Object.assign({}, options);
    if (copy.ensureAlwaysNew === undefined) {
        copy.ensureAlwaysNew = true;
    }
    if (copy.grouping === undefined || !copy.ensureAlwaysNew) {
        copy.grouping = FingerprintGrouping.NONE;
    }
    if ((copy.interval === undefined || copy.interval < MIN_SCAN_INTERVAL) &&
        !copy.ensureAlwaysNew) {
        copy.interval = MIN_SCAN_INTERVAL;
    }
    if (copy.continueOnFailure === undefined) {
        copy.continueOnFailure = false;
    }
    return copy;
}
export var FingerprintGrouping;
(function (FingerprintGrouping) {
    FingerprintGrouping[FingerprintGrouping["NONE"] = 1] = "NONE";
    FingerprintGrouping[FingerprintGrouping["INTERMEDIATE"] = 2] = "INTERMEDIATE";
    FingerprintGrouping[FingerprintGrouping["MAX"] = 4] = "MAX";
})(FingerprintGrouping || (FingerprintGrouping = {}));
function getIntervalForGrouping(grouping) {
    switch (grouping) {
        case FingerprintGrouping.NONE:
            return 30 * 1000;
        case FingerprintGrouping.INTERMEDIATE:
            return 60 * 1000;
        case FingerprintGrouping.MAX:
            return 2 * 60 * 1000;
    }
}
//# sourceMappingURL=provider.js.map